@@ -21,12 +21,11 @@
 
 #include "gcr-key-renderer.h"
 #include "gcr-display-view.h"
+#include "gcr-fingerprint.h"
 #include "gcr-icons.h"
 #include "gcr-renderer.h"
 #include "gcr-viewer.h"
 
-#include "egg/egg-hex.h"
-
 #include "gck/gck.h"
 
 #include <gdk/gdk.h>
@@ -138,7 +137,7 @@ static void
 gcr_key_renderer_init (GcrKeyRenderer *self)
 {
 	self->pv = (G_TYPE_INSTANCE_GET_PRIVATE (self, GCR_TYPE_KEY_RENDERER, GcrKeyRendererPrivate));
-	self->pv->icon = g_themed_icon_new (GCR_ICON_FOLDER);
+	self->pv->icon = g_themed_icon_new (GCR_ICON_KEY);
 }
 
 static void
@@ -241,6 +240,8 @@ gcr_key_renderer_real_render (GcrRenderer *renderer, GcrViewer *viewer)
 	GcrKeyRenderer *self;
 	GcrDisplayView *view;
 	const gchar *text = "";
+	gpointer fingerprint;
+	gsize n_fingerprint;
 	gchar *display;
 	gulong klass;
 	gulong key_type;
@@ -257,14 +258,17 @@ gcr_key_renderer_real_render (GcrRenderer *renderer, GcrViewer *viewer)
 		return;
 	}
 
-	_gcr_display_view_clear (view, renderer);
+	_gcr_display_view_begin (view, renderer);
 
-	if (!self->pv->attributes)
+	if (!self->pv->attributes) {
+		_gcr_display_view_end (view, renderer);
 		return;
+	}
 
 	if (!gck_attributes_find_ulong (self->pv->attributes, CKA_CLASS, &klass) ||
 	    !gck_attributes_find_ulong (self->pv->attributes, CKA_KEY_TYPE, &key_type)) {
 		g_warning ("private key does not have the CKA_CLASS and CKA_KEY_TYPE attributes");
+		_gcr_display_view_end (view, renderer);
 		return;
 	}
 
@@ -318,8 +322,23 @@ gcr_key_renderer_real_render (GcrRenderer *renderer, GcrViewer *viewer)
 	_gcr_display_view_append_value (view, renderer, _("Size"), display, FALSE);
 	g_free (display);
 
-	/* TODO: We need to have consistent key fingerprints. */
-	_gcr_display_view_append_value (view, renderer, _("Fingerprint"), "XX XX XX XX XX XX XX XX XX XX", TRUE);
+	/* Fingerprints */
+	_gcr_display_view_append_heading (view, renderer, _("Fingerprints"));
+
+	fingerprint = _gcr_fingerprint_from_attributes (self->pv->attributes,
+	                                                G_CHECKSUM_SHA1, &n_fingerprint);
+	if (fingerprint) {
+		_gcr_display_view_append_hex (view, renderer, _("SHA1"), fingerprint, n_fingerprint);
+		g_free (fingerprint);
+	}
+	fingerprint = _gcr_fingerprint_from_attributes (self->pv->attributes,
+	                                                G_CHECKSUM_SHA256, &n_fingerprint);
+	if (fingerprint) {
+		_gcr_display_view_append_hex (view, renderer, _("SHA256"), fingerprint, n_fingerprint);
+		g_free (fingerprint);
+	}
+
+	_gcr_display_view_end (view, renderer);
 }
 
 static void
